<?php

namespace Moblyze\FinancialPlan\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Moblyze\FinancialPlan\Http\Requests\PlanRequest;
use Moblyze\FinancialPlan\Interfaces\PlanRepositoryInterface;
use Moblyze\FinancialPlan\Models\Plan;
use Moblyze\FinancialPlan\Forms\Plan as Form;

/**
 * APIController  class for plan.
 */
class PlanAPIController extends BaseController
{

    /**
     * Initialize plan resource controller.
     *
     * @param type PlanRepositoryInterface $plan
     *
     * @return null
     */
    public function __construct(PlanRepositoryInterface $plan)
    {
        parent::__construct();
        $this->repository = $plan;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Moblyze\FinancialPlan\Repositories\Criteria\PlanResourceCriteria::class);
    }

    /**
     * Display a list of plan.
     *
     * @return Response
     */
    public function index(PlanRequest $request)
    {
        return $this->repository
            ->setPresenter(\Moblyze\FinancialPlan\Repositories\Presenter\PlanPresenter::class)
            ->paginate();
    }

    /**
     * Display plan.
     *
     * @param Request $request
     * @param Model   $plan
     *
     * @return Response
     */
    public function show(PlanRequest $request, Plan $plan)
    {
        return $plan->setPresenter(\Moblyze\FinancialPlan\Repositories\Presenter\PlanListPresenter::class);
        ;
    }

    /**
     * Create new plan.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PlanRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('financial_plan::plan.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('financial_plan/plan/' . $plan->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('financial_plan/plan');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the plan.
     *
     * @param Request $request
     * @param Model   $plan
     *
     * @return Response
     */
    public function update(PlanRequest $request, Plan $plan)
    {
        try {
            $data = $request->all();

            $plan->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('financial_plan::plan.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('financial_plan/plan/' . $plan->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('financial_plan/plan/' . $plan->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the plan.
     *
     * @param Model   $plan
     *
     * @return Response
     */
    public function destroy(PlanRequest $request, Plan $plan)
    {
        try {
            $plan->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('financial_plan::plan.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('financial_plan/plan/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('financial_plan/plan/' . $plan->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
